import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import {
  CreateAdProps,
  DefaultAdProps,
  Frequency,
} from '@modules/ad/core/domain/ad.types';
import { MarginDurationsProps } from '@modules/ad/core/domain/value-objects/margin-durations.value-object';
import { WaypointProps } from '@modules/ad/core/domain/value-objects/waypoint.value-object';

const originWaypointProps: WaypointProps = {
  position: 0,
  address: {
    houseNumber: '5',
    street: 'Avenue Foch',
    locality: 'Nancy',
    postalCode: '54000',
    country: 'France',
    coordinates: {
      lon: 48.689445,
      lat: 6.17651,
    },
  },
};
const destinationWaypointProps: WaypointProps = {
  position: 1,
  address: {
    locality: 'Paris',
    postalCode: '75000',
    country: 'France',
    coordinates: {
      lon: 48.8566,
      lat: 2.3522,
    },
  },
};
const marginDurationsProps: MarginDurationsProps = {
  mon: 600,
  tue: 600,
  wed: 600,
  thu: 600,
  fri: 600,
  sat: 600,
  sun: 600,
};
const baseCreateAdProps = {
  userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
  waypoints: [originWaypointProps, destinationWaypointProps],
};
const punctualCreateAdProps = {
  fromDate: '2023-06-21',
  toDate: '2023-06-21',
  schedule: {
    wed: '08:30',
  },
  frequency: Frequency.PUNCTUAL,
};
const recurrentCreateAdProps = {
  fromDate: '2023-06-21',
  toDate: '2024-06-20',
  schedule: {
    mon: '08:30',
    tue: '08:30',
    wed: '08:00',
    thu: '08:30',
    fri: '08:30',
  },
  frequency: Frequency.RECURRENT,
};
const punctualPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  marginDurations: marginDurationsProps,
  driver: false,
  passenger: true,
};
const recurrentPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...recurrentCreateAdProps,
  marginDurations: marginDurationsProps,
  driver: false,
  passenger: true,
};
const punctualDriverCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  marginDurations: marginDurationsProps,
  driver: true,
  passenger: false,
};
const recurrentDriverCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...recurrentCreateAdProps,
  marginDurations: marginDurationsProps,
  driver: true,
  passenger: false,
};
const punctualDriverPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  marginDurations: marginDurationsProps,
  driver: true,
  passenger: true,
};
const recurrentDriverPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...recurrentCreateAdProps,
  marginDurations: marginDurationsProps,
  driver: true,
  passenger: true,
};
const defaultAdProps: DefaultAdProps = {
  driver: false,
  passenger: true,
  marginDurations: {
    mon: 900,
    tue: 900,
    wed: 900,
    thu: 900,
    fri: 900,
    sat: 900,
    sun: 900,
  },
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
};

describe('Ad entity create', () => {
  describe('With complete props', () => {
    it('should create a new punctual passenger ad entity', async () => {
      const punctualPassengerAd: AdEntity = AdEntity.create(
        punctualPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(punctualPassengerAd.id.length).toBe(36);
      expect(punctualPassengerAd.getProps().schedule.mon).toBeUndefined();
      expect(punctualPassengerAd.getProps().schedule.wed).toBe('08:30');
      expect(punctualPassengerAd.getProps().driver).toBeFalsy();
      expect(punctualPassengerAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new punctual driver ad entity', async () => {
      const punctualDriverAd: AdEntity = AdEntity.create(
        punctualDriverCreateAdProps,
        defaultAdProps,
      );
      expect(punctualDriverAd.id.length).toBe(36);
      expect(punctualDriverAd.getProps().schedule.mon).toBeUndefined();
      expect(punctualDriverAd.getProps().schedule.wed).toBe('08:30');
      expect(punctualDriverAd.getProps().driver).toBeTruthy();
      expect(punctualDriverAd.getProps().passenger).toBeFalsy();
    });
    it('should create a new punctual driver and passenger ad entity', async () => {
      const punctualDriverPassengerAd: AdEntity = AdEntity.create(
        punctualDriverPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(punctualDriverPassengerAd.id.length).toBe(36);
      expect(punctualDriverPassengerAd.getProps().schedule.mon).toBeUndefined();
      expect(punctualDriverPassengerAd.getProps().schedule.wed).toBe('08:30');
      expect(punctualDriverPassengerAd.getProps().driver).toBeTruthy();
      expect(punctualDriverPassengerAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new recurrent passenger ad entity', async () => {
      const recurrentPassengerAd: AdEntity = AdEntity.create(
        recurrentPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(recurrentPassengerAd.id.length).toBe(36);
      expect(recurrentPassengerAd.getProps().schedule.mon).toBe('08:30');
      expect(recurrentPassengerAd.getProps().schedule.sat).toBeUndefined();
      expect(recurrentPassengerAd.getProps().driver).toBeFalsy();
      expect(recurrentPassengerAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new recurrent driver ad entity', async () => {
      const recurrentDriverAd: AdEntity = AdEntity.create(
        recurrentDriverCreateAdProps,
        defaultAdProps,
      );
      expect(recurrentDriverAd.id.length).toBe(36);
      expect(recurrentDriverAd.getProps().schedule.mon).toBe('08:30');
      expect(recurrentDriverAd.getProps().schedule.sat).toBeUndefined();
      expect(recurrentDriverAd.getProps().driver).toBeTruthy();
      expect(recurrentDriverAd.getProps().passenger).toBeFalsy();
    });
    it('should create a new recurrent driver and passenger ad entity', async () => {
      const recurrentDriverPassengerAd: AdEntity = AdEntity.create(
        recurrentDriverPassengerCreateAdProps,
        defaultAdProps,
      );
      expect(recurrentDriverPassengerAd.id.length).toBe(36);
      expect(recurrentDriverPassengerAd.getProps().schedule.mon).toBe('08:30');
      expect(
        recurrentDriverPassengerAd.getProps().schedule.sat,
      ).toBeUndefined();
      expect(recurrentDriverPassengerAd.getProps().driver).toBeTruthy();
      expect(recurrentDriverPassengerAd.getProps().passenger).toBeTruthy();
    });
  });

  describe('With incomplete props', () => {
    it('should create a new punctual passenger ad entity if no role is given', async () => {
      const punctualWithoutRoleCreateAdProps: CreateAdProps = {
        ...baseCreateAdProps,
        ...punctualCreateAdProps,
        marginDurations: marginDurationsProps,
        driver: false,
        passenger: false,
      };
      const punctualWithoutRoleAd: AdEntity = AdEntity.create(
        punctualWithoutRoleCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutRoleAd.id.length).toBe(36);
      expect(punctualWithoutRoleAd.getProps().schedule.mon).toBeUndefined();
      expect(punctualWithoutRoleAd.getProps().schedule.wed).toBe('08:30');
      expect(punctualWithoutRoleAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutRoleAd.getProps().passenger).toBeTruthy();
    });
    it('should create a new strict punctual passenger ad entity if no strict param is given', async () => {
      const punctualWithoutStrictCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: 3,
        seatsRequested: 1,
        strict: undefined,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        marginDurations: marginDurationsProps,
        driver: false,
        passenger: true,
      };
      const punctualWithoutStrictAd: AdEntity = AdEntity.create(
        punctualWithoutStrictCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutStrictAd.id.length).toBe(36);
      expect(punctualWithoutStrictAd.getProps().schedule.mon).toBeUndefined();
      expect(punctualWithoutStrictAd.getProps().schedule.wed).toBe('08:30');
      expect(punctualWithoutStrictAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutStrictAd.getProps().passenger).toBeTruthy();
      expect(punctualWithoutStrictAd.getProps().strict).toBeFalsy();
    });
    it('should create a new punctual passenger ad entity with seats requested if no seats requested param is given', async () => {
      const punctualWithoutSeatsRequestedCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: 3,
        seatsRequested: undefined,
        strict: false,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        marginDurations: marginDurationsProps,
        driver: false,
        passenger: true,
      };
      const punctualWithoutSeatsRequestedAd: AdEntity = AdEntity.create(
        punctualWithoutSeatsRequestedCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutSeatsRequestedAd.id.length).toBe(36);
      expect(
        punctualWithoutSeatsRequestedAd.getProps().schedule.mon,
      ).toBeUndefined();
      expect(punctualWithoutSeatsRequestedAd.getProps().schedule.wed).toBe(
        '08:30',
      );
      expect(punctualWithoutSeatsRequestedAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutSeatsRequestedAd.getProps().passenger).toBeTruthy();
      expect(punctualWithoutSeatsRequestedAd.getProps().seatsRequested).toBe(1);
    });
    it('should create a new punctual driver ad entity with seats proposed if no seats proposed param is given', async () => {
      const punctualWithoutSeatsProposedCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: undefined,
        seatsRequested: 1,
        strict: false,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        marginDurations: marginDurationsProps,
        driver: true,
        passenger: false,
      };
      const punctualWithoutSeatsProposedAd: AdEntity = AdEntity.create(
        punctualWithoutSeatsProposedCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutSeatsProposedAd.id.length).toBe(36);
      expect(
        punctualWithoutSeatsProposedAd.getProps().schedule.mon,
      ).toBeUndefined();
      expect(punctualWithoutSeatsProposedAd.getProps().schedule.wed).toBe(
        '08:30',
      );
      expect(punctualWithoutSeatsProposedAd.getProps().driver).toBeTruthy();
      expect(punctualWithoutSeatsProposedAd.getProps().passenger).toBeFalsy();
      expect(punctualWithoutSeatsProposedAd.getProps().seatsProposed).toBe(3);
    });
    it('should create a new punctual driver ad entity with margin durations if margin durations are empty', async () => {
      const punctualWithoutMarginDurationsCreateAdProps: CreateAdProps = {
        ...baseCreateAdProps,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        marginDurations: {},
        driver: true,
        passenger: false,
      };
      const punctualWithoutMarginDurationsAd: AdEntity = AdEntity.create(
        punctualWithoutMarginDurationsCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutMarginDurationsAd.id.length).toBe(36);
      expect(
        punctualWithoutMarginDurationsAd.getProps().schedule.mon,
      ).toBeUndefined();
      expect(punctualWithoutMarginDurationsAd.getProps().schedule.wed).toBe(
        '08:30',
      );
      expect(punctualWithoutMarginDurationsAd.getProps().driver).toBeTruthy();
      expect(punctualWithoutMarginDurationsAd.getProps().passenger).toBeFalsy();
      expect(
        punctualWithoutMarginDurationsAd.getProps().marginDurations.mon,
      ).toBe(900);
    });
    it('should create a new punctual driver ad entity with margin durations if margin durations are undefined', async () => {
      const punctualWithoutMarginDurationsCreateAdProps: CreateAdProps = {
        ...baseCreateAdProps,
        waypoints: [originWaypointProps, destinationWaypointProps],
        ...punctualCreateAdProps,
        marginDurations: undefined,
        driver: true,
        passenger: false,
      };
      const punctualWithoutMarginDurationsAd: AdEntity = AdEntity.create(
        punctualWithoutMarginDurationsCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutMarginDurationsAd.id.length).toBe(36);
      expect(
        punctualWithoutMarginDurationsAd.getProps().schedule.mon,
      ).toBeUndefined();
      expect(punctualWithoutMarginDurationsAd.getProps().schedule.wed).toBe(
        '08:30',
      );
      expect(punctualWithoutMarginDurationsAd.getProps().driver).toBeTruthy();
      expect(punctualWithoutMarginDurationsAd.getProps().passenger).toBeFalsy();
      expect(
        punctualWithoutMarginDurationsAd.getProps().marginDurations.mon,
      ).toBe(900);
    });
    it('should create a new punctual passenger ad entity with valid positions if positions are missing', async () => {
      const punctualWithoutPositionsCreateAdProps: CreateAdProps = {
        userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
        seatsProposed: 3,
        seatsRequested: 1,
        strict: undefined,
        waypoints: [
          {
            position: undefined,
            address: {
              houseNumber: '5',
              street: 'Avenue Foch',
              locality: 'Nancy',
              postalCode: '54000',
              country: 'France',
              coordinates: {
                lon: 48.689445,
                lat: 6.17651,
              },
            },
          },
          {
            position: undefined,
            address: {
              locality: 'Paris',
              postalCode: '75000',
              country: 'France',
              coordinates: {
                lon: 48.8566,
                lat: 2.3522,
              },
            },
          },
        ],
        ...punctualCreateAdProps,
        marginDurations: marginDurationsProps,
        driver: false,
        passenger: false,
      };
      const punctualWithoutPositionsAd: AdEntity = AdEntity.create(
        punctualWithoutPositionsCreateAdProps,
        defaultAdProps,
      );
      expect(punctualWithoutPositionsAd.id.length).toBe(36);
      expect(
        punctualWithoutPositionsAd.getProps().schedule.mon,
      ).toBeUndefined();
      expect(punctualWithoutPositionsAd.getProps().schedule.wed).toBe('08:30');
      expect(punctualWithoutPositionsAd.getProps().driver).toBeFalsy();
      expect(punctualWithoutPositionsAd.getProps().passenger).toBeTruthy();
      expect(punctualWithoutPositionsAd.getProps().waypoints[0].position).toBe(
        0,
      );
      expect(punctualWithoutPositionsAd.getProps().waypoints[1].position).toBe(
        1,
      );
    });
  });
});
