import { NotFoundException } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { AdMapper } from '@modules/ad/ad.mapper';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { FindAdByIdGrpcController } from '@modules/ad/interface/grpc-controllers/find-ad-by-id.grpc.controller';
import { QueryBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const mockQueryBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => '200d61a8-d878-4378-a609-c19ea71633d2')
    .mockImplementationOnce(() => {
      throw new NotFoundException();
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockAdMapper = {
  toResponse: jest.fn().mockImplementationOnce(() => ({
    userId: '8cc90d1a-4a59-4289-a7d8-078f9db7857f',
    driver: true,
    passenger: true,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-06-27',
    toDate: '2023-06-27',
    schedule: {
      tue: '07:15',
    },
    marginDurations: {
      mon: 900,
      tue: 900,
      wed: 900,
      thu: 900,
      fri: 900,
      sat: 900,
      sun: 900,
    },
    seatsProposed: 3,
    seatsRequested: 1,
    waypoints: [
      {
        position: 0,
        lon: 48.689445,
        lat: 6.17651,
        houseNumber: '5',
        street: 'Avenue Foch',
        locality: 'Nancy',
        postalCode: '54000',
        country: 'France',
      },
      {
        position: 1,
        lon: 48.8566,
        lat: 2.3522,
        locality: 'Paris',
        postalCode: '75000',
        country: 'France',
      },
    ],
  })),
};

describe('Find Ad By Id Grpc Controller', () => {
  let findAdbyIdGrpcController: FindAdByIdGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: QueryBus,
          useValue: mockQueryBus,
        },
        {
          provide: AdMapper,
          useValue: mockAdMapper,
        },
        FindAdByIdGrpcController,
      ],
    }).compile();

    findAdbyIdGrpcController = module.get<FindAdByIdGrpcController>(
      FindAdByIdGrpcController,
    );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(findAdbyIdGrpcController).toBeDefined();
  });

  it('should return an ad', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockAdMapper, 'toResponse');
    const response = await findAdbyIdGrpcController.findOnebyId({
      id: '6dcf093c-c7db-4dae-8e9c-c715cebf83c7',
    });
    expect(response.userId).toBe('8cc90d1a-4a59-4289-a7d8-078f9db7857f');
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockAdMapper.toResponse).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if ad is not found', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockAdMapper, 'toResponse');
    expect.assertions(4);
    try {
      await findAdbyIdGrpcController.findOnebyId({
        id: 'ac85f5f4-41cd-4c5d-9aee-0a1acb176fb8',
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.NOT_FOUND);
    }
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockAdMapper.toResponse).toHaveBeenCalledTimes(0);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockAdMapper, 'toResponse');
    expect.assertions(4);
    try {
      await findAdbyIdGrpcController.findOnebyId({
        id: '53c8e7ec-ef68-42bc-ba4c-5ef3effa60a6',
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockAdMapper.toResponse).toHaveBeenCalledTimes(0);
  });
});
